import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.Test
import java.time.Duration.ofMillis
import java.time.Duration.ofSeconds
import java.util.*
import java.util.concurrent.CountDownLatch

class AssertionTest {
    val calculator = Calculator();
    val person = PersonX("Jane", "Doe");
    val people = setOf(person, PersonX("John", "Doe1"))
    //׼ԣmessageͨΪʧʱ׳ʾ(ѡ)
    @Test
    fun standardAssertions() {
        assertEquals(2, calculator.add(1, 1))
        assertEquals(
            5, calculator.mul(2, 2),
            "The optional failure message is now the last parameter"
        )
        assertTrue('a' > 'b') { "Assertion messages can be lazily evaluated -- " + "to avoid constructing complex messages unnecessarily." }
    }

    //group ---- failedԶᱨԶԼеԪؽжԲ
    @Test
    fun groupedAssertions() {
        // In a grouped assertion all assertions are executed, and all failures will be reported together.
        assertAll("person",
            { assertEquals("Jane1", person.firstName) },
            { assertEquals("Doe1", person.lastName) }
        )
    }

    @Test
    fun `grouped assertions from a collection`() {
        org.junit.jupiter.api.assertAll("People with last name of Doe",
            people.map { { assertEquals("Doe1", it.lastName) } }
        )
    }

    @Test
    fun `grouped assertions from a stream`() {
        org.junit.jupiter.api.assertAll("People with first name starting with J",
            people
                .stream()
                .map {
                    // This mapping returns Stream<() -> Unit>
                    { assertTrue(it.firstName.startsWith("J")) }
                }
        )
    }
    //ͬһblockǰʧЧִ
    @Test
    fun dependentAssertions() {
        // Within a code block, if an assertion fails the subsequent code in the same block will be skipped.
        assertAll("properties",
            {
                val firstName = person.firstName;
                assertNotNull(firstName);
                // Executed only if the previous assertion is valid.
                assertAll("first name",
                    { assertTrue(firstName.startsWith("J")) },
                    { assertTrue(firstName.endsWith("e")) }
                )
            },
            {
                // Grouped assertion, so processed independently of results of first name assertions.
                val lastName = person.lastName;
                assertNotNull(lastName);
                //Executed only if the previous assertion is valid.
                assertAll("last name",
                    { assertTrue(lastName.startsWith("D")) },
                    { assertTrue(lastName.endsWith("e")) }
                )
            }
        )
    }

    //쳣ԣ
    @Test
    fun exceptionTesting() {
        val exception =
            assertThrows(ArithmeticException::class.java) { calculator.div(1, 0)}
            assertEquals("/ by zero", exception.message)
    }

    @Test
    fun `exception testing in Kotlin`() {
        val exception = org.junit.jupiter.api.assertThrows<ArithmeticException>("Should throw an exception") {
            calculator.div(1, 0)
        }
        assertEquals("/ by Zero", exception.message)
    }

    //ʱ
    @Test
    fun timeoutNotExceeded() {
        // The following assertion succeeds.
        assertTimeout(ofSeconds(2)) {
        // Perform task that takes less than 2 seconds.
            println("timeoutNotExceeded")
        }
    }

    @Test
    fun timeoutNotExceededWithResult() {
        // The following assertion succeeds, and returns the supplied object.
        val actualResult: String = assertTimeout<String>(ofSeconds(2)) { "result" }
        assertEquals("a result", actualResult)
    }

    @Test
    fun timeoutNotExceededWithMethod() {
        // The following assertion invokes a method reference and returns an object.
        val actualGreeting = assertTimeout(ofSeconds(2), ::greeting)
        assertEquals("Hello, World!", actualGreeting);
    }

    @Test
    fun timeoutExceeded() {
        // The following assertion fails with an error message similar to:
        // execution exceeded timeout of 10 ms by 91 ms
        assertTimeout(ofMillis(10)) {
            // Simulate task that takes more than 10 ms.
            Thread.sleep(100)
        }
    }

    @Test
    fun timeoutExceededWithPreemptiveTermination() {
        // The following assertion fails with an error message similar to:
        // execution timed out after 10 ms
        assertTimeoutPreemptively(ofMillis(10)) {
            // Simulate task that takes more than 10 ms.
            CountDownLatch(1).await();
        }
    }

    //ʽassertThat:֧,
    @Test
    fun testUsingAssertJ() {
        assertThat(calculator.sub(4, 1)).isEqualTo(3)
        // ӷж
        val s = "abcde"
        assertThat(s).`as`("ַж").startsWith("abc").endsWith("de").hasSize(5)
        // ж
        val i = 500
           assertThat(i).`as`("ж,ִСȽ").isGreaterThan(100).isLessThan(1000)
        // ж
        val date1 = Date()
        val date2 = Date(date1.getTime() + 100)
        val date3 = Date(date1.getTime() - 100)
        assertThat(date1).`as`("жϣڴСȽ").isBefore(date2).isAfter(date3)
        // listȽ
        val list = Arrays.asList("a", "b", "c", "d")
        assertThat(list).`as`("listβԪؼ").startsWith("a").endsWith("d").hasSize(4)
        // Mapж
        val map: MutableMap<String, Any> = HashMap()
        map["A"] = 1
        map["B"] = 2
        map["C"] = 3
        assertThat(map).`as`("Mapĳȼֵ").hasSize(3).containsKeys("A", "B", "C")
    }

    private fun greeting(): String? {
        return "Hello, World!"
    }
}